# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pytest_matrix']

package_data = \
{'': ['*']}

install_requires = \
['pytest-lazy-fixture>=0.6.3,<0.7.0', 'pytest>=5.4.3,<6.0.0']

entry_points = \
{'pytest11': ['matrix = pytest_matrix.plugin']}

setup_kwargs = {
    'name': 'pytest-matrix',
    'version': '0.7.7',
    'description': 'Provide tools for generating tests from combinations of fixtures.',
    'long_description': 'Pytest Matrix\n^^^^^^^^^^^^^\n\nEasy fixture combinations.\n\nInstalition\n-----------\n\n``pip install pytest_matrix``\n\nQuickstart:\n=============\n\n.. code:: Python\n\n    import pytest\n    from myproject import my_func\n\n        @pytest.mark.matrix(names=[\'arg_firs\'],\n                            combs=[\n                                    {\n                                       \'arg_first\': [\'val_1\', \'val_2\'],\n                                    },\n                                   ])\n        def test_my_fn(self, arg_first, arg_second, result):\n            assert my_function(arg_first, arg_second) == result\n\n\n        @pytest.fixture\n        def arg_first_val_1(self):\n            return \'val_1\'\n\n        @pytest.fixture\n        def arg_first_val_2(self):\n            return \'val_2\'\n\n        @pytest.fixture\n        def arg_second(self):\n            return \'val\'\n\n        @pytest.fixture\n        def result(self, arg_first, arg_second):\n            # prepare expected result base on arg_first and arg_second fixture\n            # in each test the arg_first parameter will have different value\n            return ...\n\n**OR**\n\n.. code:: Python\n\n    import pytest\n    from pytest_matrix import TestMatrixMixin\n    from myproject import my_func\n\n\n    class MyTestCase(TestMatrixMixin):\n        def test_my_fn(self, arg_first, arg_second, result):\n            assert my_function(arg_first, arg_second) == result\n\n        MY_FN_FIXTURES_NAMES = [\'arg_first\']\n        MY_FN_FIXTURES = [\n            {\n                \'arg_first\': [\'val_1\', \'val_2\'],\n            }\n        ]\n\n        @pytest.fixture\n        def arg_first_val_1(self):\n            return \'val_1\'\n\n        @pytest.fixture\n        def arg_first_val_2(self):\n            return \'val_2\'\n\n        @pytest.fixture\n        def arg_second(self):\n            return \'val\'\n\n        @pytest.fixture\n        def result(self, arg_first, arg_second):\n            # prepare expected result base on arg_first and arg_second fixture\n            # in each test the arg_first parameter will have different value\n            return ...\n\nThis will generate tests:\n-------------------------\n- MyTestCase::test_my_fn[arg_first_val_1|arg_second|result]\n- MyTestCase::test_my_fn[arg_first_val_2|arg_second|result]\n\n\nBoth examples are equal. But in class you have better scope controlling and other options.\n\nPlease notice **result** fixture and **arg_first** fixture. There is no **arg_first** fixture definition, the **arg_first** is created by py.test during test generation and you get access to current value of **arg_first**, same as test function receive.\n\n\n\nTest Data\n=========\n\nEvery test function must be prefixed with \'**test_**\'\nFor every test function must be defined two class attributes. If test function is named \'test_**my_function**\',\nthere must be defined **MY_FUNCTION_FIXTURES_NAMES** and **MY_FUNCTION_FIXTURES** lists.\nYou must define them in every class (they are not inherited).\n\nMY_FUNCTION_FIXTURES_NAMES:\n---------------------------\n- they are not required, it just could be little bit clearer some times, because you can choose order (the way how test name will be generated)\n- list of names of fixtures to be combined in test\n- you can define fixtures, which **ARE NOT** defined in test as parameter, these fixtures will be\n  stored in request.param and also it will be accessible by other fixtures\n\nMY_FUNCTION_FIXTURES:\n---------------------\n- list of fixture combinations\n- each combination is dict\n    - keys are same as in **MY_FUNCTION_FIXTURES_NAMES**\n    - values are list of fixture name\n        - fixture name is combination of parameter name and the list item\n\nFixtures definitions:\n---------------------\n- For every item in **MY_FUNCTION_FIXTURES** must exists fixture. It does not have to be in same class.\n- Fixtures names are defined in **MY_FUNCTION_FIXTURES**. The name si combination of key and each item in list.\n\n.. code:: Python\n\n    MY_FN_FIXTURES = [\n        {\n            \'par\': [\'a\', \'b\'],\n        }\n    ]\n    # will search for fixtures **par_a** and **par_b**\n\n\n**WARNING:**\nBe aware that every test has his own fixture context. This is useful when you want to access current value\nof function parameter by fixture name, but can be easily overlooked.\nExample:\n\n.. code:: Python\n\n    class MyTestCase(TestMatrixMixin):\n        def test_my_fn(self, par, result):\n            # some test\n\n        MY_FN_FIXTURES_NAMES = [\'par\']\n        MY_FN_FIXTURES = [\n            {\n                \'par\': [\'a\', \'b\'],\n            }\n        ]\n\n        @pytest.fixture\n        def par_a(self):\n            return \'val_a\'\n\n        @pytest.fixture\n        def par_b(self):\n            return \'val_b\'\n\n        @pytest.fixture\n        def par(self):\n            # THIS WILL NEVER BE USED IN GENERATED TESTS\n            # the context of the generated test inject in every test to par fixture either par_a or par_b\n\n        @pytest.fixture\n        def result(self, par):\n            # par is either value of par_a or par_b, it depends on test\n\n\nSimple Fixtures\n---------------\nThere are two simple fixtures types: String (with prefix \'@\') and Integer  (with prefix \'#\')\n\n.. code:: Python\n\n    import pytest\n    from myproject import my_func\n\n        @pytest.mark.matrix(names=[\'arg_firs\'],\n                            combs=[\n                                    {\n                                       \'arg_first\': [\'#1\', \'@2\'],\n                                    },\n                                   ])\n        def test_my_fn(self, arg_first):\n            assert arg_first in (1, \'2\')\n\n\nThere is no need to define fixtures arg_first_#1 (returning int(1)), arg_first_@2 (returning str(2)). It is impossible have functions (fixture definition) with these names in python anyway :).\n\n\nTest Generator\n--------------\nThe test are generated for cartesian product of defined fixture_names.\n\n.. code:: Python\n\n    class MyTestCase(TestMatrixMixin):\n        def test_my_fn(self, s, b):\n            # some test\n\n        MY_FN_FIXTURES_NAMES = [\'a\', \'b\']\n        MY_FN_FIXTURES = [\n            {\n                \'a\': [\'x\', \'y\'],\n                \'b\': [\'i\', \'j\'],\n            },\n            {\n                \'a\': [\'x\', \'y\'],\n                \'b\': [\'k\', \'l\'],\n            }\n        ]\n\nthis will generate tests:\n-------------------------\n- test_my_fn[a_x|b_i]\n- test_my_fn[a_x|b_j]\n- test_my_fn[a_y|b_i]\n- test_my_fn[a_y|b_j]\n- test_my_fn[a_x|b_k]\n- test_my_fn[a_x|b_l]\n- test_my_fn[a_y|b_k]\n- test_my_fn[a_y|b_l]\n\n\nMIXIN and inheritance\n=====================\n\nIS_MIXIN\n--------\nYou can define tests in separate class and reuse them in multiple other class. You usually don\'t want to collect these tests and run them. So you can add class attribute **IS_MIXIN = True** and tests in this class\nwill not be collected by pytest.\n\nIf you use some of these mixins you have to define **_FIXTURES** for each test. It could happen, that you won\'t use some of the tests, or you do not want generate from some of the tests.\n\nSKIP_TEST\n---------\nYou can skip tests by writing the test name in **SKIP_TESTS** class attribute.\n\nNOT_GENERATE_TESTS\n------------------\nWrite name of test you don\'t want to generate ot **NOT_GENEREATE_TESTS** attribute. Difference between NOT_GENERATE_TESTS and SKIP_TESTS is that NOT_GENERATE_TESTS will be actually run, but they will not be paramatrize.\n\nAttributes **IS_MIXIN**, **SKIP_TESTS** and **NOT_GENERATE_TESTS** are not inherited from parent class.\n\nExample:\n\n.. code:: Python\n\n    class MyTestMixin(TestMatrixMixin):\n        IS_MIXIN = True\n\n        def test_a(self):\n            pass\n\n        def test_b(self):\n            pass\n\n\n    class RealTest(MyTestMixin):\n\n        SKIP_TESTS = [\'test_a\']\n        NOT_GENERATE_TESTS = [\'test_b\']\n\n\n    class DeeperInheritanceTest(RealTest):\n        SKIP_TESTS = [\'test_b\']\n\n        A_FIXTURES_NAMES = [\'par\']\n        A_FIXTURES = [\n            {\n                \'par\': [\'a\', \'b\'],\n            }\n        ]\n\n        @pytest.fixture\n        def par_a(self):\n            return \'val_a\'\n\n        @pytest.fixture\n        def par_b(self):\n            return \'val_b\'\n\n\nThis will skip:\n---------------\n- RealTest.test_a\n- DeeperInheritanceTest.test_b\n\nAnd run these tests:\n--------------------\n- RealTest.test_b\n- DeeperInheritanceTest.test_a[par_a]\n- DeeperInheritanceTest.test_a[par_b]\n\n\nCombination Tester\n==================\nSometimes you want test if you covered all combinations of specific fixtures. You can define the combinations you want to cover in class attribute **COMBINATIONS_COVER**.\n\ntest_combcover_fn_fx_x_y PASSED\n-------------------------------\n.. code:: python\n\n    class TestCombinations(TestMatrixMixin):\n        FN_FIXTURES = [\n            {\n                \'x\': [\'a\', \'b\'],\n                \'y\': [\'c\'],\n            },\n            {\n                \'x\': [\'a\'],\n                \'y\': [\'d\'],\n            }\n        ]\n        FN_FIXTURES_NAMES = [\'x\', \'y\']\n\n        FX_FIXTURES = [\n            {\n                \'x\': [\'b\'],\n                \'y\': [\'d\'],\n                \'z\': [\'j\', \'k\']\n            }\n        ]\n        FX_FIXTURES_NAMES = [\'x\', \'y\', \'z\']\n\n        # **COMBINATIONS**\n        COMBINATIONS_COVER = [\n            {\n                "fixture_names": [\'x\', \'y\'],\n                "fixture_functions": [\'fn\', \'fx\'],\n            }\n        ]\n\n        def test_fx(self):\n            pass\n\n        def test_fn(self):\n            pass\n\n        @pytest.fixture\n        def x_a(self):\n            pass\n\n        #... rest of class with rest of fixtures (x_b, y_c, y_d, z_j, z_k)\n\nThis will generate test **test_combcover_fn_fx_x_y**. The prefix for combination cover test is **test_combcover_** followed by names of functions (*test_fx* and *test_fn*) separated by underscore: **fn_fx_** and suffix are names of fixtures (their combinations we want to cover) **x_y**.\n\nThis concrete test will find all types of **x** *(\'a\', \'b\')* and **y** *(\'c\', \'d\')* fixtures, combine them *([x_a|y_c], [x_b|y_c], [x_a|y_d], [x_b|y_d])* and compare them with combinations manually defined in **_FIXTURES** configuration *(fn: [x_a|y_c], [x_b|y_c], [x_a|y_d] and fx: [x_b|y_d])*. If they are not equal, the test will fail and print all uncovered combinations. But this test will pass.\n\n\ntest_combcover_fn_x_y FAILED\n-------------------------------\nNow we added other test combination.\n\n.. code:: python\n\n    class TestCombinations(TestMatrixMixin):\n        FN_FIXTURES = [\n            {\n                \'x\': [\'a\', \'b\'],\n                \'y\': [\'c\'],\n            },\n            {\n                \'x\': [\'a\'],\n                \'y\': [\'d\'],\n            }\n        ]\n\n        # other configs\n\n        COMBINATIONS_COVER = [\n            {\n                "fixture_names": [\'x\', \'y\'],\n                "fixture_functions": [\'fn\', \'fx\'],\n            },\n            {\n                "fixture_names": [\'x\', \'y\'],\n                "fixture_functions": [\'fn\'],  # **TEST ONLY ONE TEST\'S FIXTURE COMBINATIONS**\n            },\n        ]\n\n        # rest of the class...\n\nThis will generate two tests **test_combcover_fn_fx_x_y** *PASSED* and **test_combcover_fn_x_y** *FAILED*. The second test failed because combination of *[x_b|y_d]* is not covered in **FN_FIXTURES**. It will be also shown in test_result.\n\ntest_combcover_fx_x_y FAILED OR PASSED according to scope\n---------------------------------------------------------\n\nThere are two type of scopes which combcover can use when looking for all types of fixtures.\n    - *class* scope:\n        - default scope\n        - the combcover will look in ALL **_FIXTURES** defined in same class\n    - *functions* scope:\n        - the combcover will look for fixture types only in these **_FIXTURES** from functions define in combcover config\n\n.. code:: python\n\n    class TestCombinations(TestMatrixMixin):\n        FN_FIXTURES = [\n            {\n                \'x\': [\'a\', \'b\'],\n                \'y\': [\'c\'],\n            },\n            {\n                \'x\': [\'a\'],\n                \'y\': [\'d\'],\n            }\n        ]\n        FN_FIXTURES_NAMES = [\'x\', \'y\']\n\n        FX_FIXTURES = [\n            {\n                \'x\': [\'b\'],\n                \'y\': [\'d\'],\n                \'z\': [\'j\', \'k\']\n            }\n        ]\n        FX_FIXTURES_NAMES = [\'x\', \'y\', \'z\']\n\n        # **COMBINATIONS**\n        COMBINATIONS_COVER = [\n            {\n                "fixture_names": [\'x\', \'y\'],\n                "fixture_functions": [\'fx\'],\n                "scope": \'class\',  # this is not required *class* is default scope\n            }\n        ]\n        # rest of the class...\n\n\nThe test will find all types of **x** *(\'a\', \'b\')* and **y** *(\'c\', \'d\')* in **ALL** fixtures, combine them *([x_a|y_c], [x_b|y_c], [x_a|y_d], [x_b|y_d])* and compare them with combinations manually defined in **FX_FIXTURES** configuration *([x_b|y_d])*. The result of the test will be **FAILED** and missing combinations will be: *[x_a|y_c], [x_b|y_c], [x_a|y_d]*\n\nIf you remove the *scope* key from **COMBINATIONS_COVER** the test will be **PASSED**, because combcover will be looking for only for fixtures type defined in **FX_FIXTURES** *(x_a and y_d)*.\n\n.. code:: python\n\n    class TestCombinations(TestMatrixMixin):\n        FN_FIXTURES = [\n            {\n                \'x\': [\'a\', \'b\'],\n                \'y\': [\'c\'],\n            },\n            {\n                \'x\': [\'a\'],\n                \'y\': [\'d\'],\n            }\n        ]\n        FN_FIXTURES_NAMES = [\'x\', \'y\']\n\n        FX_FIXTURES = [\n            {\n                \'x\': [\'b\'],\n                \'y\': [\'d\'],\n                \'z\': [\'j\', \'k\']\n            }\n        ]\n        FX_FIXTURES_NAMES = [\'x\', \'y\', \'z\']\n\n        # **COMBINATIONS**\n        COMBINATIONS_COVER = [\n            {\n                "fixture_names": [\'x\', \'y\'],\n                "fixture_functions": [\'fx\'],\n                "scope": \'functions\',  # this is required\n            }\n        ]\n        # rest of the class...\n\nThis combocover test will PASS\n\n\nTODO:\n=====\n[X] exclude test if test\'s cls TestMatrixMixin.is_mixin == True\n[X] force to define _FIXTURES and _FIXTURES_NAMES in every class, except mixin class\n[X] raise error if _FIXTURES keys are not exactly same as _FIXTURE_NAMES\n[ ] edit function to control use of all fixtures combinations\n[X] check names of fixtures combinations are same as defined FIXTURES_NAMES\n[X] allow skip tests\n[X] allow not generate tests\n[ ] validate sctructure of SKIP_TESTS, NOT_GENERATE_TESTS, FIXTURE_NAMES and FIXTURES\n[ ] check for duplicity in _FIXTURES and COMBINATION_COVER\n',
    'author': 'dburton90',
    'author_email': 'daniel.barton@seznam.cz',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/dburton90/pytest_matrix.git',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
